# Validate 
def square_param_validator(args):
    if len(args) == 2:
        return True
    else:
        return False
    
def rectangle_param_validator(args):
    if len(args) == 3:
        return True
    else:
        return False
    
def circle_param_validator(args):
    if len(args) == 2:
        return True
    else:
        return False
    
def triangle_param_validator(args):
    if len(args) == 2:
        return True
    elif len(args) == 3:
        return True
    elif len(args) == 4:
        return True
    else:
        return False
    
def parallelogram_param_validator(args):
    if len(args) == 3:
        return True
    else:
        return False
    
def rhombus_param_validator(args):
    if len(args) == 2:
        return True
    else:
        return False
    
def trapezium_param_validator(args):
    if len(args) == 5:
        return True
    else:
        return False
    
def ellipse_param_validator(args):
    if len(args) == 3:
        return True
    else:
        return False
    
def cube_param_validator(args):
    if len(args) == 2:
        return True
    else:
        return False
    
def cuboid_param_validator(args):
    if len(args) == 4:
        return True
    else:
        return False
    
def sphere_param_validator(args):
    if len(args) == 2:
        return True
    else:
        return False
    
def cylinder_param_validator(args):
    if len(args) == 2:
        return True
    elif len(args) == 3:
        return True
    else:
        return False
    
def cone_param_validator(args):
    if len(args) == 2:
        return True
    elif len(args) == 3:
        return True
    else:
        return False
    
def pyramid_param_validator(args):
    if len(args) == 2:
        return True
    elif len(args) == 3:
        return True
    else:
        return False
    
def hemisphere_param_validator(args):
    if len(args) == 2:
        return True
    else:
        return False