from setuptools import setup, find_packages

setup(
    name="mensurapy",
    version="0.1",
    description="This package provides a comprehensive set of functions to calculate the area and perimeter of 2D shapes, as well as the surface area and volume of 3D geometric shapes. It supports various units of measurement and is designed to simplify geometric computations for students, educators, and developers alike.",
    long_description="",
    long_description_content_type='text/markdown',
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.12",
        "Operating System :: OS Independent",
    ],
    install_requires=[],  # List dependencies here
    author="Anuva Das",
    author_email="anuvadas@hotmail.com",
    url="https://github.com/Anuva13/MensuraPy.git",
    license="MIT",
)